# File:  testTurboAPIClient.rb
 
require File.dirname(__FILE__) + "/../main/turboAPIClient"
require File.dirname(__FILE__) + "/../main/mail"
require "test/unit"
require "restclient.rb"

#TODO add test cases like python and parametrizzare to, from, subject, html_content...etc
class TestTurboAPIClient < Test::Unit::TestCase
  USERNAME_OK   = "username@turbo-smtp.com"     #insert TurboSMTP Account "username"
  USERNAME_NOT_OK = "username@turbo-smtp.co"    #insert invalid username here
  PASSWORD_OK   = "insercorretpasswordhere"                #insert TurboSMTP Account "password"
  PASSWORD_NOT_OK   = "insertincorrectpasswordhere"
  SERVER_URL_OK = "https://api.turbo-smtp.com/api/mail/send"
  SERVER_URL_NOT_OK = "https://api.turbo-smtp.com/api/mail/s"
  FROM_MAIL_FIELD           = "correctfrom@email.com"  #insert email "from" value
  INCORRECT_FROM_MAIL_FIELD = "an_incorrect_@email_address_from"
  TO_MAIL_FIELD             = "correctto@email.com"  #insert email "to" value
  INCORRECT_TO_MAIL_FIELD   = "an_incorrect_@email_address_to"
  SUBJECT_CONTENT           = "Ruby Test TurboAPI Send(email) : campo SUBJECT"
  HTML_CONTENT              = "Ruby Test TurboAPI Send(email) : campo HTML_CONTENT"
 
     
  @@turboAPIClient = TurboAPIClient.new
  
  def test_messageOK  
    mail = Mail.new 
    mail.from = FROM_MAIL_FIELD
    mail.to = TO_MAIL_FIELD
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    
    returnedMessage = TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail)
    if ('{"message":"OK"}' == returnedMessage) 
      assert(true, "");
    elsif ('{"message":"error","errors":{"message":"error","errors":"nocredit"}}'  == returnedMessage) 
      assert(true, "")
    else 
      assert(false, "Unknown returned message!!")
    end    
  end
  
  #case: "from" parameter is missing  -->The mail is not send
  def test_message_From_Param_Not_Found
    mail = Mail.new 
    mail.from = ""
    mail.to = TO_MAIL_FIELD
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    assert_equal('{"message":"error","errors":["missing or not valid sender email (from)"]}', TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail) )
  end
  
  #case: "from" parameter is incorrect  -->The mail is not send
  def test_message_From_Param_Incorrect
      mail = Mail.new
      mail.from = INCORRECT_FROM_MAIL_FIELD
      mail.to = TO_MAIL_FIELD
      mail.subject = SUBJECT_CONTENT
      mail.html_content = HTML_CONTENT 
      assert_equal('{"message":"error","errors":["missing or not valid sender email (from)"]}', TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail) ) 
  end
  
  #case: "to" parameter is missing  -->The mail is not send
  def test_message_To_Param_Not_Found
    mail = Mail.new 
    mail.from = FROM_MAIL_FIELD
    mail.to = ""
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    assert_equal('{"message":"error","errors":["missing recipients (to)"]}',  TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail) )
  end
  
  #case: "to" parameter is incorrect  -->The mail is not send
  def test_message_To_Param_Incorrect
      mail = Mail.new 
      mail.from = FROM_MAIL_FIELD
      mail.to = INCORRECT_TO_MAIL_FIELD
      mail.subject = SUBJECT_CONTENT
      mail.html_content = HTML_CONTENT 
      assert_equal('{"message":"error","errors":[["\'' + INCORRECT_TO_MAIL_FIELD + '\' \'to\' email not valid"]]}', TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail) ) 
  end

  #case: "from" and "to" parameter are missing  -->The mail is not send
  def test_message_From_And_To_Param_Not_Found
    mail = Mail.new 
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    assert_equal('{"message":"error","errors":["missing or not valid sender email (from)","missing recipients (to)"]}',  TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail) )
  end
 
  #case: "from" and "to" parameter are incorrect  -->The mail is not send
  def test_message_From_And_To_Param_Incorrect
    mail = Mail.new 
    mail.from = INCORRECT_FROM_MAIL_FIELD
    mail.to = INCORRECT_TO_MAIL_FIELD
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    assert_equal('{"message":"error","errors":["missing or not valid sender email (from)",["\'' + INCORRECT_TO_MAIL_FIELD + '\' \'to\' email not valid"]]}', TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_OK, mail) ) 
  end

  #case: incorrect authentication credentials (username)
  def test_error412_wrong_user
    mail = Mail.new 
    mail.from = FROM_MAIL_FIELD
    mail.to = TO_MAIL_FIELD
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    assert_raise( RestClient::PreconditionFailed ) {  TurboAPIClient.new.sendMail(USERNAME_NOT_OK, PASSWORD_OK,  SERVER_URL_OK, mail) }
  end
  
  #case: incorrect authentication credentials(password)
  def test_error412_wrong_password
      mail = Mail.new 
      mail.from = FROM_MAIL_FIELD
      mail.to = TO_MAIL_FIELD
      mail.subject = SUBJECT_CONTENT
      mail.html_content = HTML_CONTENT 
      assert_raise( RestClient::PreconditionFailed ) {  TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_NOT_OK,  SERVER_URL_OK, mail) }
    end
  
  #case: incorrect resource URL or resource not found
  def test_error404
    mail = Mail.new 
    mail.from = FROM_MAIL_FIELD
    mail.to = TO_MAIL_FIELD
    mail.subject = SUBJECT_CONTENT
    mail.html_content = HTML_CONTENT 
    assert_raise( RestClient::ResourceNotFound ) {  TurboAPIClient.new.sendMail(USERNAME_OK, PASSWORD_OK,  SERVER_URL_NOT_OK, mail) }
  end

end
